@@ -1,18 +1,17 @@
 /*
  * The main header file for SIP.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
  * This copy of SIP may also used under the terms of the GNU General Public
  * License v2 or v3 as published by the Free Software Foundation which can be
- * found in the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this
- * package.
- * 
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -25,6 +24,13 @@
 #include <sys/types.h>
 
 
+/*
+ * Define the SIP version number.
+ */
+#define SIP_VERSION         0x040c01
+#define SIP_VERSION_STR     "4.12.1"
+
+
 #ifdef TRUE
 #undef TRUE
 #endif
@@ -54,6 +60,7 @@
 #define MOD_IS_CONSOLIDATED     0x0002  /* It is a consolidated module. */
 #define MOD_IS_COMPOSITE        0x0004  /* It is a composite module. */
 #define MOD_IS_TRANSFORMED      0x0008  /* It's types have been transformed. */
+#define MOD_USE_ARG_NAMES       0x0010  /* Use real argument names. */
 
 #define hasDelayedDtors(m)  ((m)->modflags & MOD_HAS_DELAYED_DTORS)
 #define setHasDelayedDtors(m)   ((m)->modflags |= MOD_HAS_DELAYED_DTORS)
@@ -64,6 +71,8 @@
 #define isContainer(m)      ((m)->modflags & (MOD_IS_CONSOLIDATED | MOD_IS_COMPOSITE))
 #define setIsTransformed(m) ((m)->modflags |= MOD_IS_TRANSFORMED)
 #define isTransformed(m)    ((m)->modflags & MOD_IS_TRANSFORMED)
+#define setUseArgNames(m)   ((m)->modflags |= MOD_USE_ARG_NAMES)
+#define useArgNames(m)      ((m)->modflags & MOD_USE_ARG_NAMES)
 
 
 /* Handle section flags. */
@@ -100,6 +109,7 @@
 #define CLASS_CANNOT_COPY   0x10000000  /* It cannot be copied. */
 #define CLASS_CANNOT_ASSIGN 0x20000000  /* It cannot be assigned. */
 #define CLASS_ALLOW_NONE    0x40000000  /* The class will handle None. */
+#define CLASS_HAS_NONLAZY   0x80000000  /* The class has non-lazy methods. */
 
 #define hasSigSlots(cd)     ((cd)->classflags & CLASS_HAS_SIGSLOTS)
 #define setHasSigSlots(cd)  ((cd)->classflags |= CLASS_HAS_SIGSLOTS)
@@ -150,6 +160,8 @@
 #define setCannotAssign(cd) ((cd)->classflags |= CLASS_CANNOT_ASSIGN)
 #define classHandlesNone(cd)    ((cd)->classflags & CLASS_ALLOW_NONE)
 #define setClassHandlesNone(cd) ((cd)->classflags |= CLASS_ALLOW_NONE)
+#define hasNonlazyMethod(cd)    ((cd)->classflags & CLASS_HAS_NONLAZY)
+#define setHasNonlazyMethod(cd) ((cd)->classflags |= CLASS_HAS_NONLAZY)
 
 #define isPublicDtor(cd)    ((cd)->classflags & SECT_IS_PUBLIC)
 #define setIsPublicDtor(cd) ((cd)->classflags |= SECT_IS_PUBLIC)
@@ -167,7 +179,6 @@
 #define CTOR_HOLD_GIL       0x00000800  /* The ctor holds the GIL. */
 #define CTOR_XFERRED        0x00001000  /* Ownership is transferred. */
 #define CTOR_IS_DEPRECATED  0x00002000  /* The ctor is deprecated. */
-#define CTOR_KEYWORD_ARGS   0x00004000  /* The ctor allows keyword arguments. */
 
 #define isPublicCtor(c)     ((c)->ctorflags & SECT_IS_PUBLIC)
 #define setIsPublicCtor(c)  ((c)->ctorflags |= SECT_IS_PUBLIC)
@@ -187,8 +198,6 @@
 #define setIsResultTransferredCtor(c)   ((c)->ctorflags |= CTOR_XFERRED)
 #define isDeprecatedCtor(c) ((c)->ctorflags & CTOR_IS_DEPRECATED)
 #define setIsDeprecatedCtor(c)  ((c)->ctorflags |= CTOR_IS_DEPRECATED)
-#define useKeywordArgsCtor(c)   ((c)->ctorflags & CTOR_KEYWORD_ARGS)
-#define setUseKeywordArgsCtor(c)    ((c)->ctorflags |= CTOR_KEYWORD_ARGS)
 
 
 /* Handle member flags. */
@@ -204,8 +213,8 @@
 #define setNoArgParser(m)   ((m)->memberflags |= MEMBR_NO_ARG_PARSER)
 #define notVersioned(m)     ((m)->memberflags & MEMBR_NOT_VERSIONED)
 #define setNotVersioned(m)  ((m)->memberflags |= MEMBR_NOT_VERSIONED)
-#define useKeywordArgsFunction(m)   ((m)->memberflags & MEMBR_KEYWORD_ARGS)
-#define setUseKeywordArgsFunction(m)    ((m)->memberflags |= MEMBR_KEYWORD_ARGS)
+#define useKeywordArgs(m)   ((m)->memberflags & MEMBR_KEYWORD_ARGS)
+#define setUseKeywordArgs(m)    ((m)->memberflags |= MEMBR_KEYWORD_ARGS)
 
 
 /* Handle enum flags.  These are combined with the section flags. */
@@ -255,8 +264,9 @@
 #define OVER_IS_GLOBAL      0x00400000  /* It is a global operator. */
 #define OVER_IS_COMPLEMENTARY   0x00800000  /* It is a complementary operator. */
 #define OVER_IS_DEPRECATED  0x01000000  /* It is deprecated. */
-#define OVER_KEYWORD_ARGS   0x02000000  /* It allows keyword arguments. */
-#define OVER_REALLY_PROT    0x04000000  /* It really is protected. */
+#define OVER_REALLY_PROT    0x02000000  /* It really is protected. */
+#define OVER_IS_DELATTR     0x04000000  /* It is __delattr__. */
+#define OVER_RAISES_PY_EXC  0x08000000  /* It raises a Python exception. */
 
 #define isPublic(o)         ((o)->overflags & SECT_IS_PUBLIC)
 #define setIsPublic(o)      ((o)->overflags |= SECT_IS_PUBLIC)
@@ -266,8 +276,10 @@
 #define setIsPrivate(o)     ((o)->overflags |= SECT_IS_PRIVATE)
 #define isSlot(o)           ((o)->overflags & SECT_IS_SLOT)
 #define setIsSlot(o)        ((o)->overflags |= SECT_IS_SLOT)
+#define resetIsSlot(o)      ((o)->overflags &= ~SECT_IS_SLOT)
 #define isSignal(o)         ((o)->overflags & SECT_IS_SIGNAL)
 #define setIsSignal(o)      ((o)->overflags |= SECT_IS_SIGNAL)
+#define resetIsSignal(o)    ((o)->overflags &= ~SECT_IS_SIGNAL)
 
 #define isVirtual(o)        ((o)->overflags & OVER_IS_VIRTUAL)
 #define setIsVirtual(o)     ((o)->overflags |= OVER_IS_VIRTUAL)
@@ -305,10 +317,12 @@
 #define setIsComplementary(o)   ((o)->overflags |= OVER_IS_COMPLEMENTARY)
 #define isDeprecated(o)     ((o)->overflags & OVER_IS_DEPRECATED)
 #define setIsDeprecated(o)  ((o)->overflags |= OVER_IS_DEPRECATED)
-#define useKeywordArgs(o)   ((o)->overflags & OVER_KEYWORD_ARGS)
-#define setUseKeywordArgs(o)    ((o)->overflags |= OVER_KEYWORD_ARGS)
 #define isReallyProtected(o)    ((o)->overflags & OVER_REALLY_PROT)
 #define setIsReallyProtected(o) ((o)->overflags |= OVER_REALLY_PROT)
+#define isDelattr(o)        ((o)->overflags & OVER_IS_DELATTR)
+#define setIsDelattr(o)     ((o)->overflags |= OVER_IS_DELATTR)
+#define raisesPyException(o)    ((o)->overflags & OVER_RAISES_PY_EXC)
+#define setRaisesPyException(o) ((o)->overflags |= OVER_RAISES_PY_EXC)
 
 
 /* Handle variable flags. */
@@ -339,6 +353,7 @@
 #define ARG_SINGLE_SHOT     0x1000  /* The slot is only ever fired once. */
 #define ARG_RESULT_SIZE     0x2000  /* It defines the result size. */
 #define ARG_KEEP_REF        0x4000  /* Keep a reference. */
+#define ARG_NO_COPY         0x8000  /* Disable copying of const references. */
 
 #define isReference(a)      ((a)->argflags & ARG_IS_REF)
 #define setIsReference(a)   ((a)->argflags |= ARG_IS_REF)
@@ -372,6 +387,8 @@
 #define setResultSize(a)    ((a)->argflags |= ARG_RESULT_SIZE)
 #define keepReference(a)    ((a)->argflags & ARG_KEEP_REF)
 #define setKeepReference(a) ((a)->argflags |= ARG_KEEP_REF)
+#define noCopy(a)           ((a)->argflags & ARG_NO_COPY)
+#define setNoCopy(a)        ((a)->argflags |= ARG_NO_COPY)
 
 
 /* Handle name flags. */
@@ -417,8 +434,22 @@
 #define setNoTypeName(td)   ((td)->tdflags |= TD_NO_TYPE_NAME)
 
 
-/* Slot types. */
+/* Warning categories. */
+typedef enum {
+    ParserWarning,
+    DeprecationWarning
+} Warning;
 
+
+/* Levels of keyword argument support. */
+typedef enum {
+    NoKwArgs = 0,
+    AllKwArgs,
+    OptionalKwArgs
+} KwArgs;
+
+
+/* Slot types. */
 typedef enum {
     str_slot,
     unicode_slot,
@@ -476,6 +507,8 @@ typedef enum {
     index_slot,
     iter_slot,
     next_slot,
+    setattr_slot,
+    delattr_slot,       /* This is local to the parser. */
     no_slot
 } slotType;
 
@@ -533,7 +566,10 @@ typedef enum {
     ssize_type,
     ascii_string_type,
     latin1_string_type,
-    utf8_string_type
+    utf8_string_type,
+    byte_type,
+    sbyte_type,
+    ubyte_type
 } argType;
 
 
@@ -581,7 +617,7 @@ typedef struct {
 /* A version qualifier. */
 
 typedef struct _qualDef {
-    char *name;                         /* The qualifier name. */
+    const char *name;                   /* The qualifier name. */
     qualType qtype;                     /* The qualifier type. */
     struct _moduleDef *module;          /* The defining module. */
     int line;                           /* Timeline if it is a time. */
@@ -718,6 +754,7 @@ typedef struct _moduleDef {
     apiVersionRangeDef *api_versions;   /* The defined APIs. */
     apiVersionRangeDef *api_ranges;     /* The list of API version ranges. */
     int modflags;                       /* The module flags. */
+    KwArgs kwargs;                      /* The styleof keyword argument support. */
     int qobjclass;                      /* QObject class, -1 if none. */
     struct _memberDef *othfuncs;        /* List of other functions. */
     struct _overDef *overs;             /* Global overloads. */
@@ -732,8 +769,10 @@ typedef struct _moduleDef {
     codeBlock *initcode;                /* Initialisation code. */
     codeBlock *postinitcode;            /* Post-initialisation code. */
     codeBlock *unitcode;                /* Compilation unit code. */
+    codeBlock *unitpostinccode;         /* Compilation unit post-include code. */
+    codeBlock *docstring;               /* The docstring. */
     int parts;                          /* The number of parts generated. */
-    char *file;                         /* The filename. */
+    const char *file;                   /* The filename. */
     qualDef *qualifiers;                /* The list of qualifiers. */
     argDef *types;                      /* The array of numbered types. */
     int nrtypes;                        /* The number of numbered types. */
@@ -749,6 +788,7 @@ typedef struct _moduleDef {
     struct _ifaceFileList *used;        /* Interface files used. */
     struct _moduleListDef *allimports;  /* The list of all imports. */
     struct _moduleListDef *imports;     /* The list of direct imports. */
+    struct _autoPyNameDef *autopyname;  /* The Python naming rules. */
     struct _moduleDef *next;            /* Next in the list. */
 } moduleDef;
 
@@ -796,6 +836,7 @@ typedef struct _mappedTypeDef {
     ifaceFileDef *iff;                  /* The interface file. */
     struct _memberDef *members;         /* The static member functions. */
     struct _overDef *overs;             /* The static overloads. */
+    codeBlock *typecode;                /* Type code. */
     codeBlock *convfromcode;            /* Convert from C++ code. */
     codeBlock *convtocode;              /* Convert to C++ code. */
     struct _mappedTypeDef *next;        /* Next in the list. */
@@ -868,11 +909,23 @@ typedef struct _varDef {
 } varDef;
 
 
+/* A property definition. */
+
+typedef struct _propertyDef {
+    nameDef *name;                      /* The property name. */
+    const char *get;                    /* The name of the getter method. */
+    const char *set;                    /* The name of the setter method. */
+    codeBlock *docstring;               /* The docstring. */
+    struct _propertyDef *next;          /* Next in the list. */
+} propertyDef;
+
+
 /* An overloaded member function definition. */
 
 typedef struct _overDef {
     char *cppname;                      /* The C++ name. */
     int overflags;                      /* The overload flags. */
+    KwArgs kwargs;                      /* The keyword argument support. */
     struct _memberDef *common;          /* Common parts. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
@@ -890,6 +943,7 @@ typedef struct _overDef {
 
 typedef struct _ctorDef {
     int ctorflags;                      /* The ctor flags. */
+    KwArgs kwargs;                      /* The keyword argument support. */
     apiVersionRangeDef *api_range;      /* The optional API version range. */
     signatureDef pysig;                 /* The Python signature. */
     signatureDef *cppsig;               /* The C++ signature, NULL if /NoDerived/. */
@@ -983,7 +1037,7 @@ typedef struct _mroDef {
 /* A class definition. */
 
 typedef struct _classDef {
-    int classflags;                     /* The class flags. */
+    unsigned classflags;                /* The class flags. */
     int pyqt4_flags;                    /* The PyQt4 specific flags. */
     nameDef *pyname;                    /* The Python name. */
     ifaceFileDef *iff;                  /* The interface file. */
@@ -1018,6 +1072,7 @@ typedef struct _classDef {
     codeBlock *segcountcode;            /* Segment count code (Python v2). */
     codeBlock *charbufcode;             /* Character buffer code (Python v2). */
     codeBlock *picklecode;              /* Pickle code. */
+    propertyDef *properties;            /* The properties. */
     struct _classDef *next;             /* Next in the list. */
 } classDef;
 
@@ -1040,6 +1095,32 @@ typedef struct _mappedTypeTmplDef {
 } mappedTypeTmplDef;
 
 
+/* The extracts for an identifier. */
+
+typedef struct _extractDef {
+    const char *id;                     /* The identifier. */
+    struct _extractPartDef *parts;      /* The ordered list of parts. */
+    struct _extractDef *next;           /* The next in the list. */
+} extractDef;
+
+
+/* Part of an extract for an identifier. */
+
+typedef struct _extractPartDef {
+    int order;                          /* The order of the part. */
+    codeBlock *part;                    /* The part itself. */
+    struct _extractPartDef *next;       /* The next in the list. */
+} extractPartDef;
+
+
+/* A rule for automatic Python naming. */
+
+typedef struct _autoPyNameDef {
+    const char *remove_leading;         /* Leading string to remove. */
+    struct _autoPyNameDef *next;        /* The next in the list. */
+} autoPyNameDef;
+
+
 /* The parse tree corresponding to the specification file. */
 
 typedef struct {
@@ -1060,6 +1141,7 @@ typedef struct {
     int sigslots;                       /* Set if signals or slots are used. */
     int genc;                           /* Set if we are generating C code. */
     struct _stringList *plugins;        /* The list of plugins. */
+    struct _extractDef *extracts;       /* The list of extracts. */
 } sipSpec;
 
 
@@ -1084,18 +1166,22 @@ extern char *sipVersion;                /* The version of SIP. */
 extern stringList *includeDirList;      /* The include directory list for SIP files. */
 
 
-void parse(sipSpec *, FILE *, char *, stringList *, stringList *, int, int);
+void parse(sipSpec *, FILE *, char *, stringList *, stringList *, KwArgs, int);
 void parserEOF(char *,parserContext *);
 void transform(sipSpec *);
 void generateCode(sipSpec *, char *, char *, char *, const char *, int, int,
-        int, int, stringList *, const char *, int);
+        int, int, stringList *, const char *, int, int);
+void generateExtracts(sipSpec *pt, const stringList *extracts);
+void addExtractPart(sipSpec *pt, const char *id, int order, codeBlock *part);
 void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile);
 void generateXML(sipSpec *pt, moduleDef *mod, const char *xmlFile);
 void generateExpression(valueDef *vd, int in_str, FILE *fp);
-void warning(char *,...);
+void warning(Warning w, const char *fmt, ...);
+void deprecated(const char *msg);
 void fatal(char *,...);
 void fatalScopedName(scopedNameDef *);
 int setInputFile(FILE *open_fp, parserContext *pc, int optional);
+void resetLexerState();
 void *sipMalloc(size_t n);
 void *sipCalloc(size_t nr, size_t n);
 char *sipStrdup(const char *);
@@ -1119,9 +1205,11 @@ void prOverloadName(FILE *fp, overDef *od);
 void prOverloadDecl(FILE *fp, ifaceFileDef *scope, overDef *od, int defval);
 void prScopedPythonName(FILE *fp, classDef *scope, const char *pyname);
 int prPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd, int sec,
-        int names, int defaults, int in_str);
+        int names, int defaults, int in_str, int is_signal);
 void searchTypedefs(sipSpec *pt, scopedNameDef *snd, argDef *ad);
+int isZeroArgSlot(memberDef *md);
 int isIntReturnSlot(memberDef *md);
+int isSSizeReturnSlot(memberDef *md);
 int isLongReturnSlot(memberDef *md);
 int isVoidReturnSlot(memberDef *md);
 int isNumberSlot(memberDef *md);
@@ -1139,6 +1227,7 @@ void yywarning(char *);
 nameDef *cacheName(sipSpec *pt, const char *name);
 scopedNameDef *encodedTemplateName(templateDef *td);
 apiVersionRangeDef *findAPI(sipSpec *pt, const char *name);
+memberDef *findMethod(classDef *cd, const char *name);
 
 
 /* These are only here because bison publically references them. */
@@ -1154,11 +1243,12 @@ typedef enum {
     opt_name_flag,
     dotted_name_flag,
     integer_flag,
+    opt_integer_flag,
     api_range_flag
 } flagType;
 
 typedef struct {
-    char *fname;                        /* The flag name. */
+    const char *fname;                  /* The flag name. */
     flagType ftype;                     /* The flag type. */
     union {                             /* The flag value. */
         char *sval;                     /* A string value. */
@@ -1172,4 +1262,128 @@ typedef struct {
     optFlag flags[MAX_NR_FLAGS];        /* Each flag. */
 } optFlags;
 
+
+/* These represent the configuration of different directives. */
+
+/* %API */
+typedef struct _apiCfg {
+    int token;
+    const char *name;
+    int version;
+} apiCfg;
+
+/* %AutoPyName */
+typedef struct _autoPyNameCfg {
+    int token;
+    const char *remove_leading;
+} autoPyNameCfg;
+
+/* %CompositeModule */
+typedef struct _compModuleCfg {
+    int token;
+    const char *name;
+    codeBlock *docstring;
+} compModuleCfg;
+
+/* %ConsolidatedModule */
+typedef struct _consModuleCfg {
+    int token;
+    const char *name;
+    codeBlock *docstring;
+} consModuleCfg;
+
+/* %DefaultEncoding */
+typedef struct _defEncodingCfg {
+    int token;
+    const char *name;
+} defEncodingCfg;
+
+/* %DefaultMetatype */
+typedef struct _defMetatypeCfg {
+    int token;
+    const char *name;
+} defMetatypeCfg;
+
+/* %DefaultSupertype */
+typedef struct _defSupertypeCfg {
+    int token;
+    const char *name;
+} defSupertypeCfg;
+
+/* %Exception */
+typedef struct _exceptionCfg {
+    int token;
+    codeBlock *type_header_code;
+    codeBlock *raise_code;
+} exceptionCfg;
+
+/* %Extract */
+typedef struct _extractCfg {
+    int token;
+    const char *id;
+    int order;
+} extractCfg;
+
+/* %Feature */
+typedef struct _featureCfg {
+    int token;
+    const char *name;
+} featureCfg;
+
+/* %Import */
+typedef struct _importCfg {
+    int token;
+    const char *name;
+} importCfg;
+
+/* %Include */
+typedef struct _includeCfg {
+    int token;
+    const char *name;
+    int optional;
+} includeCfg;
+
+/* %License */
+typedef struct _licenseCfg {
+    int token;
+    const char *type;
+    const char *licensee;
+    const char *signature;
+    const char *timestamp;
+} licenseCfg;
+
+/* %Module */
+typedef struct _moduleCfg {
+    int token;
+    int c_module;
+    KwArgs kwargs;
+    const char *name;
+    int use_arg_names;
+    int version;
+    codeBlock *docstring;
+} moduleCfg;
+
+/* %Plugin */
+typedef struct _pluginCfg {
+    int token;
+    const char *name;
+} pluginCfg;
+
+/* %Property */
+typedef struct _propertyCfg {
+    int token;
+    const char *get;
+    const char *name;
+    const char *set;
+    codeBlock *docstring;
+} propertyCfg;
+
+/* Variable sub-directives. */
+typedef struct _variableCfg {
+    int token;
+    codeBlock *access_code;
+    codeBlock *get_code;
+    codeBlock *set_code;
+} variableCfg;
+
 #endif
