@@ -1,18 +1,17 @@
 /*
  * The parse tree transformation module for SIP.
  *
- * Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
- * 
+ * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+ *
  * This file is part of SIP.
- * 
+ *
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- * 
+ *
  * This copy of SIP may also used under the terms of the GNU General Public
  * License v2 or v3 as published by the Free Software Foundation which can be
- * found in the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this
- * package.
- * 
+ * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
+ *
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
@@ -66,6 +65,7 @@ static void resolvePySigTypes(sipSpec *,moduleDef *,classDef *,overDef *,signatu
 static void resolveVariableType(sipSpec *,varDef *);
 static void fatalNoDefinedType(scopedNameDef *);
 static void getBaseType(sipSpec *,moduleDef *,classDef *,argDef *);
+static void resolveType(sipSpec *,moduleDef *,classDef *,argDef *, int);
 static void searchClassScope(sipSpec *,classDef *,scopedNameDef *,argDef *);
 static void searchMappedTypes(sipSpec *,moduleDef *,scopedNameDef *,argDef *);
 static void searchEnums(sipSpec *,scopedNameDef *,argDef *);
@@ -89,6 +89,8 @@ static void resolveInstantiatedClassTemplate(sipSpec *pt, argDef *type);
 static void setStringPoolOffsets(sipSpec *pt);
 static const char *templateString(const char *src, scopedNameDef *names,
         scopedNameDef *values);
+static mappedTypeDef *copyTemplateType(mappedTypeDef *mtd, argDef *ad);
+static void checkProperties(classDef *cd);
 
 
 /*
@@ -282,9 +284,12 @@ void transform(sipSpec *pt)
             cd->iff->ifacenr = cd->real->iff->ifacenr;
     }
 
-    /* Mark classes that can have an assignment helper. */
+    /* Additional class specific checks. */
     for (cd = pt->classes; cd != NULL; cd = cd->next)
+    {
         checkAssignmentHelper(pt, cd);
+        checkProperties(cd);
+    }
 
     setStringPoolOffsets(pt);
 }
@@ -515,8 +520,13 @@ static void checkAssignmentHelper(sipSpec *pt, classDef *cd)
         if (ct->cppsig == NULL || !isPublicCtor(ct))
             continue;
 
-        if (ct->cppsig->nrArgs == 0)
+        if (ct->cppsig->nrArgs == 0 || ct->cppsig->args[0].defval != NULL)
+        {
+            /*
+             * The ctor either has no arguments or all arguments have defaults.
+             */
             pub_def_ctor = TRUE;
+        }
         else if (ct->cppsig->nrArgs == 1)
         {
             argDef *ad = &ct->cppsig->args[0];
@@ -635,13 +645,6 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
             /* Previous error checking means this will always work. */
             dcd = findAltClassImplementation(pt, al->arg.u.mtd);
 
-        /*
-         * If the destination class is in a different module then use
-         * a proxy.
-         */
-        if (dcd->iff->module != mod)
-            dcd = getProxy(mod, dcd);
-
         /* Create the new ctor. */
         ct = sipMalloc(sizeof (ctorDef));
 
@@ -659,6 +662,16 @@ static void moveClassCasts(sipSpec *pt, moduleDef *mod, classDef *cd)
         ad->defval = NULL;
         ad->u.cd = cd;
 
+        /*
+         * If the destination class is in a different module then use
+         * a proxy.
+         */
+        if (dcd->iff->module != mod)
+        {
+            ifaceFileIsUsed(&mod->used, ad);
+            dcd = getProxy(mod, dcd);
+        }
+
         ifaceFileIsUsed(&dcd->iff->used, ad);
 
         ct->pysig.nrArgs = 1;
@@ -870,11 +883,14 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
             *mdhead = md;
         }
 
-        /* Move the overload. */
+        /* Move the overload to the end of the destination list. */
         setIsPublic(od);
         setIsGlobal(od);
         od->common = md;
-        od->next = *odhead;
+        od->next = NULL;
+
+        while (*odhead != NULL)
+            odhead = &(*odhead)->next;
 
         *odhead = od;
 
@@ -888,6 +904,10 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
             *arg0 = *arg1;
             od->pysig.nrArgs = 1;
         }
+
+        /* Remove the only argument of unary operators. */
+        if (isZeroArgSlot(md))
+            od->pysig.nrArgs = 0;
     }
 }
 
@@ -1368,14 +1388,23 @@ static void transformCtors(sipSpec *pt, classDef *cd)
 
         /*
          * Now check that the Python signature doesn't conflict with an
-         * earlier one.
+         * earlier one.  If there is %MethodCode then assume that it will
+         * handle any potential conflicts.
          */
-        for (prev = cd->ctors; prev != ct; prev = prev->next)
-            if (samePythonSignature(&prev->pysig, &ct->pysig))
+        if (ct->methodcode == NULL)
+        {
+            for (prev = cd->ctors; prev != ct; prev = prev->next)
             {
-                fatalScopedName(classFQCName(cd));
-                fatal(" has ctors with the same Python signature\n");
+                if (prev->methodcode != NULL)
+                    continue;
+
+                if (samePythonSignature(&prev->pysig, &ct->pysig))
+                {
+                    fatalScopedName(classFQCName(cd));
+                    fatal(" has ctors with the same Python signature\n");
+                }
             }
+        }
 
         if (isDeprecatedClass(cd))
             setIsDeprecatedCtor(ct);
@@ -1515,35 +1544,42 @@ static void transformScopeOverloads(sipSpec *pt, classDef *c_scope,
 
         /*
          * Now check that the Python signature doesn't conflict with an earlier
-         * one.
+         * one.  If there is %MethodCode then assume that it will handle any
+         * potential conflicts.
          */
-        for (prev = overs; prev != od; prev = prev->next)
+        if (od->methodcode == NULL)
         {
-            if (prev->common != od->common)
-                continue;
-
-            /* They can only conflict if one is unversioned. */
-            if (prev->api_range != NULL && od->api_range != NULL)
-                continue;
-
-            if (samePythonSignature(&prev->pysig, &od->pysig))
+            for (prev = overs; prev != od; prev = prev->next)
             {
-                ifaceFileDef *iff;
+                if (prev->common != od->common)
+                    continue;
 
-                if (mt_scope != NULL)
-                    iff = mt_scope->iff;
-                else if (c_scope != NULL)
-                    iff = c_scope->iff;
-                else
-                    iff = NULL;
+                if (prev->methodcode != NULL)
+                    continue;
 
-                if (iff != NULL)
+                /* They can only conflict if one is unversioned. */
+                if (prev->api_range != NULL && od->api_range != NULL)
+                    continue;
+
+                if (samePythonSignature(&prev->pysig, &od->pysig))
                 {
-                    fatalScopedName(iff->fqcname);
-                    fatal("::");
-                }
+                    ifaceFileDef *iff;
 
-                fatal("%s() has overloaded functions with the same Python signature\n", od->common->pyname->text);
+                    if (mt_scope != NULL)
+                        iff = mt_scope->iff;
+                    else if (c_scope != NULL)
+                        iff = c_scope->iff;
+                    else
+                        iff = NULL;
+
+                    if (iff != NULL)
+                    {
+                        fatalScopedName(iff->fqcname);
+                        fatal("::");
+                    }
+
+                    fatal("%s() has overloaded functions with the same Python signature\n", od->common->pyname->text);
+                }
             }
         }
 
@@ -1797,7 +1833,7 @@ static void resolveMappedTypeTypes(sipSpec *pt, mappedTypeDef *mt)
 
         /* Leave templates as they are. */
         if (ad->atype != template_type)
-            getBaseType(pt, mt->iff->module, NULL, ad);
+            resolveType(pt, mt->iff->module, NULL, ad, TRUE);
     }
 
     /* Make sure that the signature result won't cause problems. */
@@ -1860,9 +1896,16 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     /* Handle the Python signature. */
     resolvePySigTypes(pt, mod, c_scope, od, &od->pysig, isSignal(od));
 
-    /* These slots must return int. */
     res = &od->pysig.result;
 
+    /* These slots must return SIP_SSIZE_T (or int - deprecated). */
+    if (isSSizeReturnSlot(od->common))
+        if ((res->atype != ssize_type && res->atype != int_type) || res->nrderefs != 0 ||
+            isReference(res) || isConstArg(res))
+            fatal("%s slots must return SIP_SSIZE_T\n",
+                    od->common->pyname->text);
+
+    /* These slots must return int. */
     if (isIntReturnSlot(od->common))
         if (res->atype != int_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
@@ -1942,7 +1985,7 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
                     fatal("::");
                 }
 
-                fatal("%s() unsupported signal argument type\n");
+                fatal("%s() unsupported signal argument type\n", od->cppname);
             }
         }
         else if (!supportedType(scope,od,ad,TRUE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL || (isVirtual(od) && od -> virthandler -> virtcode == NULL)))
@@ -2008,6 +2051,9 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case enum_type:
     case bool_type:
     case cbool_type:
+    case byte_type:
+    case sbyte_type:
+    case ubyte_type:
     case ushort_type:
     case short_type:
     case uint_type:
@@ -2017,6 +2063,7 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     case long_type:
     case ulonglong_type:
     case longlong_type:
+    case ssize_type:
     case pyobject_type:
     case pytuple_type:
     case pylist_type:
@@ -2137,6 +2184,9 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case enum_type:
     case bool_type:
     case cbool_type:
+    case byte_type:
+    case sbyte_type:
+    case ubyte_type:
     case ushort_type:
     case short_type:
     case uint_type:
@@ -2146,6 +2196,7 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case long_type:
     case ulonglong_type:
     case longlong_type:
+    case ssize_type:
     case pyobject_type:
     case pytuple_type:
     case pylist_type:
@@ -2155,6 +2206,12 @@ static int supportedType(classDef *cd,overDef *od,argDef *ad,int outputs)
     case pytype_type:
         if (isReference(ad))
         {
+            if (isConstArg(ad))
+            {
+                ensureInput(cd, od, ad);
+                return TRUE;
+            }
+
             if (ad -> nrderefs == 0 && outputs)
             {
                 defaultOutput(ad);
@@ -2421,12 +2478,14 @@ int sameSignature(signatureDef *sd1,signatureDef *sd2,int strict)
             (t) == latin1_string_type || (t) == utf8_string_type)
 #define pyAsFloat(t)    ((t) == cfloat_type || (t) == float_type || \
             (t) == cdouble_type || (t) == double_type)
-#define pyAsInt(t)  ((t) == bool_type || \
+#define pyAsInt(t)  ((t) == bool_type || (t) == ssize_type || \
+            (t) == byte_type || (t) == sbyte_type || (t) == ubyte_type || \
             (t) == short_type || (t) == ushort_type || \
             (t) == cint_type || (t) == int_type || (t) == uint_type)
 #define pyAsLong(t) ((t) == long_type || (t) == longlong_type)
 #define pyAsULong(t)    ((t) == ulong_type || (t) == ulonglong_type)
 #define pyAsAuto(t) ((t) == bool_type || \
+            (t) == byte_type || (t) == sbyte_type || (t) == ubyte_type || \
             (t) == short_type || (t) == ushort_type || \
             (t) == int_type || (t) == uint_type || \
             (t) == float_type || (t) == double_type)
@@ -2515,6 +2574,12 @@ int sameBaseType(argDef *a1, argDef *a2)
         if (a1->atype == defined_type && a2->atype == mapped_type)
             return compareScopedNames(a1->u.snd, a2->u.mtd->iff->fqcname) == 0;
 
+        if (a1->atype == enum_type && a2->atype == defined_type)
+            return compareScopedNames(a1->u.ed->fqcname, a2->u.snd) == 0;
+
+        if (a1->atype == defined_type && a2->atype == enum_type)
+            return compareScopedNames(a1->u.snd, a2->u.ed->fqcname) == 0;
+
         return FALSE;
     }
 
@@ -2604,7 +2669,6 @@ static int samePythonSignature(signatureDef *sd1, signatureDef *sd2)
     }
 
     return (a1 < 0 && a2 < 0);
-
 }
 
 
@@ -2764,6 +2828,16 @@ static void scopeDefaultValue(sipSpec *pt,classDef *cd,argDef *ad)
  */
 static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
         argDef *type)
+{
+    resolveType(pt, mod, c_scope, type, FALSE);
+}
+
+
+/*
+ * Resolve a type if possible.
+ */
+static void resolveType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
+        argDef *type, int allow_defined)
 {
     /* Loop until we've got to a base type. */
     while (type->atype == defined_type)
@@ -2788,7 +2862,15 @@ static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
             searchClasses(pt, mod, snd, type);
 
         if (type->atype == no_type)
+        {
+            if (allow_defined)
+            {
+                type->atype = defined_type;
+                return;
+            }
+
             fatalNoDefinedType(snd);
+        }
     }
 
     /* Get the base type of any slot arguments. */
@@ -2897,6 +2979,8 @@ static mappedTypeDef *instantiateMappedTypeTemplate(sipSpec *pt, moduleDef *mod,
     if (type_values != NULL)
         freeScopedName(type_values);
 
+    mtd = copyTemplateType(mtd, type);
+
     return mtd;
 }
 
@@ -3050,6 +3134,8 @@ static void searchMappedTypes(sipSpec *pt, moduleDef *context,
                     continue;
             }
 
+            mtd = copyTemplateType(mtd, ad);
+
             /* Copy the type. */
             ad->atype = mapped_type;
             ad->u.mtd = mtd;
@@ -3066,6 +3152,50 @@ static void searchMappedTypes(sipSpec *pt, moduleDef *context,
 }
 
 
+/*
+ * If a mapped type is based on a template then create a copy that keeps the
+ * original types of the template arguments.
+ */
+static mappedTypeDef *copyTemplateType(mappedTypeDef *mtd, argDef *ad)
+{
+    int a;
+    signatureDef *src, *dst;
+    mappedTypeDef *mtd_copy;
+
+    /* There is no need to do anything for non-template types. */
+    if (mtd->type.atype != template_type)
+        return mtd;
+
+    /* Retain the original types if there are any. */
+    mtd_copy = mtd;
+    src = &ad->u.td->types;
+
+    for (a = 0; a < src->nrArgs; ++a)
+    {
+        typedefDef *tdd = src->args[a].original_type;
+
+        if (tdd != NULL)
+        {
+            /*
+             * Create the copy now that we know it is needed and if it hasn't
+             * already been done.
+             */
+            if (mtd_copy == mtd)
+            {
+                mtd_copy = sipMalloc(sizeof (mappedTypeDef));
+                *mtd_copy = *mtd;
+
+                dst = &mtd_copy->type.u.td->types;
+            }
+
+            dst->args[a].original_type = tdd;
+        }
+    }
+
+    return mtd_copy;
+}
+
+
 /*
  * Search the typedefs for a name and return the type.
  */
@@ -3427,3 +3557,38 @@ static int generatingCodeForModule(sipSpec *pt, moduleDef *mod)
 
     return (pt->module == mod);
 }
+
+
+/*
+ * Check that any properties are valid.
+ */
+static void checkProperties(classDef *cd)
+{
+    propertyDef *pd;
+
+    for (pd = cd->properties; pd != NULL; pd = pd->next)
+    {
+        if (findMethod(cd, pd->get) == NULL)
+            fatal("Property %s.%s has no get method %s()\n", cd->pyname->text,
+                    pd->name->text, pd->get);
+
+        if (pd->set != NULL && findMethod(cd, pd->set) == NULL)
+            fatal("Property %s.%s has no set method %s()\n", cd->pyname->text,
+                    pd->name->text, pd->set);
+    }
+}
+
+
+/*
+ * Return the method of a class with a given name.
+ */
+memberDef *findMethod(classDef *cd, const char *name)
+{
+    memberDef *md;
+
+    for (md = cd->members; md != NULL; md = md->next)
+        if (strcmp(md->pyname->text, name) == 0)
+            break;
+
+    return md;
+}
